@@ -81,9 +81,9 @@ class Link:
         # length is mile or km
         self.length = length
         self.lanes = lanes
-        # 1:one direction, 2:two way
+        # 1: one direction, 2: two way
         self.type = link_type
-        # length:km, free_speed: km/h
+        # free_speed: mile/h or km/h
         self.free_flow_travel_time_in_min = (
             length / max(SMALL_DIVISOR, free_speed) * 60
         )
@@ -101,14 +101,6 @@ class Link:
         self.travel_time_by_period = [0] * demand_period_size
         self.flow_vol_by_period = [0] * demand_period_size
         self.vdfperiods = []
-        # Peiheng, 04/05/21, not needed for the current implementation
-        # self.queue_length_by_slot = [0] * MAX_TIME_PERIODS
-        # self.vol_by_period_by_at = [
-        #     [0] * demand_period_size for i in range(agent_type_size)
-        # ]
-        # self.travel_marginal_cost_by_period = [
-        #     [0] * demand_period_size for i in range(agent_type_size)
-        # ]
 
     def get_link_id(self):
         return self.id
@@ -166,26 +158,23 @@ class Link:
     def reset_period_flow_vol(self, tau):
         self.flow_vol_by_period[tau] = 0
 
-    # def reset_period_agent_vol(self, tau, agent_type):
-    #     self.vol_by_period_by_at[tau][agent_type] = 0
-
     def increase_period_flow_vol(self, tau, fv):
         self.flow_vol_by_period[tau] += fv
 
-    # def increase_period_agent_vol(self, tau, agent_type, v):
-    #     self.vol_by_period_by_at[tau][agent_type] += v
-
-    def calculate_td_vdf(self):
-        for tau in range(self.demand_period_size):
-            self.travel_time_by_period[tau] = (
-                self.vdfperiods[tau].run_bpr(self.flow_vol_by_period[tau])
-            )
-
-    # Peiheng, 04/05/21, not needed for the current implementation
-    # def calculate_agent_marginal_cost(self, tau, agent_type):
-    #     self.travel_marginal_cost_by_period[tau][agent_type.get_id()] = (
-    #         self.vdfperiods[tau].marginal_base * agent_type.get_pce()
-    #     )
+    def calculate_td_vdf(self, demand_periods=None, iter_num=None):
+        if demand_periods is None or iter_num is None:
+            for tau in range(self.demand_period_size):
+                self.travel_time_by_period[tau] = (
+                    self.vdfperiods[tau].run_bpr(self.flow_vol_by_period[tau])
+                )
+        else:
+            for dp in demand_periods:
+                tau = dp.get_id()
+                reduction_ratio = dp.get_reduction_ratio(self.id, iter_num)
+                self.travel_time_by_period[tau] = (
+                    self.vdfperiods[tau].run_bpr(self.flow_vol_by_period[tau],
+                                                 reduction_ratio)
+                )
 
 
 class Agent:
@@ -212,10 +201,6 @@ class Agent:
         # Passenger Car Equivalent (PCE) of the agent
         self.PCE_factor = 1
         self.path_cost = 0
-        # self.departure_time_in_simu_interval = int(
-        #     self.departure_time_in_min
-        #     * 60 /_NUM_OF_SECS_PER_SIMU_INTERVAL
-        #     + 0.5)
         self.b_generated = False
         self.b_complete_trip = False
         self.feasible_path_exist_flag = False
@@ -244,9 +229,6 @@ class Agent:
     def get_node_path(self):
         return self.node_path
 
-    # def get_dep_simu_intvl(self):
-    #     return self.departure_time_in_simu_interval
-
 
 class Network:
 
@@ -386,31 +368,8 @@ class Network:
                 agent_id += 1
                 agent_no += 1
 
-                # step 3: update the g_simulation_start_time_in_min and
-                # g_simulation_end_time_in_min
-                # if agent.departure_time_in_min < g_simulation_start_time_in_min:
-                #     g_simulation_start_time_in_min = agent.departure_time_in_min
-                # if agent.departure_time_in_min > g_simulation_end_time_in_min:
-                #     g_simulation_end_time_in_min = agent.departure_time_in_min
-
-                #step 4: add the agent to the time dependent agent list
-                # departure_time = agent.get_dep_simu_intvl()
-                # if departure_time not in self.agent_td_list_dict.keys():
-                #     self.agent_td_list_dict[departure_time] = []
-                # self.agent_td_list_dict[departure_time].append(
-                #     agent.get_seq_no()
-                # )
-
                 self.agent_list.append(agent)
 
-        # 03/22/21, comment out until departure time is enabled
-        # in the future release
-
-        #step 3.6:sort agents by the departure time
-        # agents.sort(key=lambda agent: agent.departure_time_in_min)
-        # for i, agent in enumerate(agents):
-        #     agent.agent_seq_no = i
-
         self.agent_size = len(self.agent_list)
         print(f'the number of agents is {self.agent_size}')
 
@@ -720,12 +679,28 @@ class AgentType:
         return self.ffs
 
 
+class SpecialEvent:
+
+    def __init__(self, name, beg_iteration, end_iteration) -> None:
+        self.name = name
+        self.beg_iter = beg_iteration
+        self.end_iter = end_iteration
+        self.affected_links = {}
+
+    def get_beg_iteration(self):
+        return self.beg_iter
+
+    def get_end_iteration(self):
+        return self.end_iter
+
+
 class DemandPeriod:
 
     def __init__(self, id=0, period='AM', time_period='0700_0800'):
         self.id = id
         self.period = period
         self.time_period = time_period
+        self.special_event = None
 
     def get_id(self):
         return self.id
@@ -733,6 +708,23 @@ class DemandPeriod:
     def get_period(self):
         return self.period
 
+    def get_beg_iteration(self):
+        return self.special_event.get_beg_iteration()
+
+    def get_end_iteration(self):
+        return self.special_event.get_end_iteration()
+
+    def get_reduction_ratio(self, link_id, iter_num):
+        if self.special_event is None:
+            return 1
+
+        if iter_num < self.get_beg_iteration() - 1 or iter_num > self.get_end_iteration() - 1:
+            return 1
+
+        try:
+            return self.special_event.affected_links[link_id]
+        except KeyError:
+            return 1
 
 class Demand:
 
@@ -770,7 +762,6 @@ class VDFPeriod:
         self.fftt = fftt
         self.capacity = cap
         self.phf = phf
-        self.marginal_base = 1
         self.avg_travel_time = 0
         self.voc = 0
 
@@ -783,17 +774,9 @@ class VDFPeriod:
     def get_fftt(self):
         return self.fftt
 
-    def run_bpr(self, vol):
+    def run_bpr(self, vol, reduction_ratio=1):
         vol = max(0, vol)
-        self.voc = vol / max(SMALL_DIVISOR, self.capacity)
-
-        self.marginal_base = (
-            self.fftt
-            * self.alpha
-            * self.beta
-            * pow(self.voc, self.beta - 1)
-        )
-
+        self.voc = vol / max(SMALL_DIVISOR, self.capacity * reduction_ratio)
         self.avg_travel_time = (
             self.fftt
             + self.fftt
